#!/bin/bash

BN=`basename $0`

cn=`hostname --fqdn 2>/dev/null` ||  cn=`hostname`
bits=2048
days=1
httpdconf=/tmp/cert.$$
dummy=dummy-
subdir=/conf

usage () {
    echo "Usage:"
    echo "  $BN [-h HOST] [-b BITS] [-d DAYS] [-c CONF_DIR] [-r]"
    echo
    echo "Options:"
    echo " -h host: certificate host name (default: $cn)"
    echo " -d days: certificate validation period in days (default: $days)"
    echo " -b bits: key size in bits (default: $bits)"
    echo " -c dir:  store certificates in apache conf directory (default: $httpdconf)"
    echo " -r:      create certificate named server.crt (not dummy-server.crt)"
    echo "Example:"
    echo "  $BN -d 365 -b 4092 -c /etc/httpd"
    exit 1
}

while getopts h:b:d:c:r c ; do
  case $c in
    h) cn=$OPTARG ;;
    b) bits=$OPTARG ;;
    d) days=$OPTARG ;;
    c) httpdconf=$OPTARG ;;
    r) dummy= ; subdir= ;;
    *) usage ;;
  esac
done

openssl version || exit 1

host=( $(tr '.' '\n' <<< $cn | tac) )
host[1]=${host[1]:-MyCompany}
host[2]=${host[2]:-MyDepartment}

algo=RSA
conf="$httpdconf$subdir/ssl"

mkdir -m 755 -p "$conf".crt "$conf".csr "$conf".key || exit 1

# exit if file(s) exist

for name in server ca ; do
  for file in crt csr key ; do
    ls -l "$conf.$file"/$name.$file 2>/dev/null && {
      echo "(existing file found, exiting.)"; exit 1
    }
  done
done

# find some random files

unset randfiles
for file in /var/log/messages /etc/hosts /etc/group /etc/resolv.conf /bin/ls; do
    [ -r $file ] && randfiles="$randfiles:$file"
done

cfgfile=$httpdconf/.mkcert.cfg
randfiles="${randfiles#:}"
export RANDFILE=$httpdconf/.mkcert.rnd

# create_req_cfg

create_req_cfg() {
  local country=${host[0]}
  [[ ${#country} = 2 ]] || country=US
cat > $cfgfile <<!
[ req ]
default_bits            = $bits
distinguished_name      = req_DN
prompt                  = no
[ req_DN ]
countryName             = $country
stateOrProvinceName     = ${host[1]}
localityName            = ${host[2]}
organizationName        = ${host[1]}
organizationalUnitName  = $1
commonName              = $cn
emailAddress            = $1@${host[1]}.${host[0]}
!
}

# create_x509_cfg

create_x509_cfg() {
            cat > $cfgfile <<!
extensions = x509v3
[ x509v3 ]
subjectAltName   = email:copy
nsComment        = "mod_ssl generated test server certificate"
nsCertType       = $1
!
[ $1 = sslCA ] && cat >> $cfgfile <<!
basicConstraints = CA:true,pathlen:0
!
}

# create CA certificate

openssl genrsa -rand $randfiles -out "$conf".key/${dummy}ca.key $bits || exit 1

create_req_cfg CA

openssl req -config $cfgfile -new -key "$conf".key/${dummy}ca.key -out "$conf".csr/${dummy}ca.csr || exit 1

create_x509_cfg sslCA

openssl x509 -extfile $cfgfile -days $days -signkey "$conf".key/${dummy}ca.key -in "$conf".csr/${dummy}ca.csr -req -out "$conf".crt/${dummy}ca.crt || exit 1

# create server certificate

openssl genrsa -rand $randfiles -out "$conf".key/${dummy}server.key $bits || exit 1

create_req_cfg Server

openssl req -config $cfgfile -new -key "$conf".key/${dummy}server.key -out "$conf".csr/${dummy}server.csr || exit 1

create_x509_cfg server

openssl x509 -extfile $cfgfile -days $days -set_serial $(date +%s) -CA "$conf".crt/${dummy}ca.crt -CAkey "$conf".key/${dummy}ca.key -in "$conf".csr/${dummy}server.csr -req -out "$conf".crt/${dummy}server.crt || exit 1

chmod 600 $httpdconf$subdir/ssl.*/${dummy}*
chmod 644 $httpdconf$subdir/ssl.crt/${dummy}*

echo

find $httpdconf$subdir/ssl.* -type f -exec ls \-l {} \;
